@@ -34,6 +34,8 @@ enum act_from {
 	ACT_F_HTTP_REQ,    /* http-request */
 	ACT_F_HTTP_RES,    /* http-response */
 	ACT_F_TCP_CHK,     /* tcp-check. */
+	ACT_F_CFG_PARSER,  /* config parser */
+	ACT_F_CLI_PARSER,  /* command line parser */
 };
 
 enum act_return {
@@ -66,9 +68,9 @@ enum act_flag {
 
 
 /* known actions to be used without any action function pointer. This enum is
- * typically used in a switch case, iff .action_ptr is undefined. So if an
- * action function is defined for one of following action types, the function
- * have the priority over the switch.
+ * typically used in a switch case, if and only if .action_ptr is undefined. So
+ * if an action function is defined for one of following action types, the
+ * function have the priority over the switch.
  */
 enum act_name {
 	ACT_CUSTOM = 0,
@@ -93,6 +95,26 @@ enum act_name {
 	ACT_TCP_CLOSE, /* close at the sender's */
 };
 
+/* Timeout name valid for a set-timeout rule */
+enum act_timeout_name {
+	ACT_TIMEOUT_SERVER,
+	ACT_TIMEOUT_TUNNEL,
+};
+
+enum act_normalize_uri {
+	ACT_NORMALIZE_URI_PATH_MERGE_SLASHES,
+	ACT_NORMALIZE_URI_PATH_STRIP_DOT,
+	ACT_NORMALIZE_URI_PATH_STRIP_DOTDOT,
+	ACT_NORMALIZE_URI_PATH_STRIP_DOTDOT_FULL,
+	ACT_NORMALIZE_URI_QUERY_SORT_BY_NAME,
+	ACT_NORMALIZE_URI_PERCENT_TO_UPPERCASE,
+	ACT_NORMALIZE_URI_PERCENT_TO_UPPERCASE_STRICT,
+	ACT_NORMALIZE_URI_PERCENT_DECODE_UNRESERVED,
+	ACT_NORMALIZE_URI_PERCENT_DECODE_UNRESERVED_STRICT,
+	ACT_NORMALIZE_URI_FRAGMENT_STRIP,
+	ACT_NORMALIZE_URI_FRAGMENT_ENCODE,
+};
+
 /* NOTE: if <.action_ptr> is defined, the referenced function will always be
  *       called regardless the action type. */
 struct act_rule {
@@ -105,16 +127,16 @@ struct act_rule {
 	                              struct session *sess, struct stream *s, int opts);
 	int (*check_ptr)(struct act_rule *rule, struct proxy *px, char **err); /* ptr to check function */
 	void (*release_ptr)(struct act_rule *rule); /* ptr to release function */
-	struct action_kw *kw;
+	const struct action_kw *kw;
 	struct applet applet;                  /* used for the applet registration. */
 	union {
 		struct {
 			struct sample_expr *expr;
 			char *varname;
 			char *resolvers_id;
-			struct dns_resolvers *resolvers;
-			struct dns_options *dns_opts;
-		} dns;                         /* dns resolution */
+			struct resolvers *resolvers;
+			struct resolv_options *opts;
+		} resolv;                      /* resolving */
 		struct {
 			int i;                 /* integer param (status, nice, loglevel, ..) */
 			struct ist str;        /* string param (reason, header name, ...) */
@@ -137,6 +159,11 @@ struct act_rule {
 			struct sample_expr *expr;
 			int idx;
 		} capid;
+		struct {
+			int value;                  /* plain timeout value in ms if no expr is used */
+			enum act_timeout_name type; /* timeout type */
+			struct sample_expr *expr;   /* timeout value as an expression */
+		} timeout;
 		struct hlua_rule *hlua_rule;
 		struct {
 			struct sample_expr *expr;
@@ -162,7 +189,7 @@ struct action_kw {
 	const char *kw;
 	enum act_parse_ret (*parse)(const char **args, int *cur_arg, struct proxy *px,
 	                            struct act_rule *rule, char **err);
-	int match_pfx;
+	int flags;
 	void *private;
 };
 
