@@ -24,10 +24,13 @@
 
 #include <stdio.h>
 #include <haproxy/action-t.h>
+#include <haproxy/cfgparse.h>
 #include <haproxy/list.h>
+#include <haproxy/sample.h>
 
-int act_resolution_cb(struct dns_requester *requester, struct dns_nameserver *nameserver);
-int act_resolution_error_cb(struct dns_requester *requester, int error_code);
+int act_resolution_cb(struct resolv_requester *requester, struct dns_counters *counters);
+int act_resolution_error_cb(struct resolv_requester *requester, int error_code);
+const char *action_suggest(const char *word, const struct list *keywords, const char **extra);
 
 static inline struct action_kw *action_lookup(struct list *keywords, const char *kw)
 {
@@ -39,10 +42,10 @@ static inline struct action_kw *action_lookup(struct list *keywords, const char
 
 	list_for_each_entry(kw_list, keywords, list) {
 		for (i = 0; kw_list->kw[i].kw != NULL; i++) {
-			if (kw_list->kw[i].match_pfx &&
+			if ((kw_list->kw[i].flags & KWF_MATCH_PREFIX) &&
 			    strncmp(kw, kw_list->kw[i].kw, strlen(kw_list->kw[i].kw)) == 0)
 				return &kw_list->kw[i];
-			if (!strcmp(kw, kw_list->kw[i].kw))
+			if (strcmp(kw, kw_list->kw[i].kw) == 0)
 				return &kw_list->kw[i];
 		}
 	}
@@ -62,7 +65,7 @@ static inline void action_build_list(struct list *keywords,
 	end = p + chk->size - 1;
 	list_for_each_entry(kw_list, keywords, list) {
 		for (i = 0; kw_list->kw[i].kw != NULL; i++) {
-			l = snprintf(p, end - p, "'%s%s', ", kw_list->kw[i].kw, kw_list->kw[i].match_pfx ? "(*)" : "");
+			l = snprintf(p, end - p, "'%s%s', ", kw_list->kw[i].kw, (kw_list->kw[i].flags & KWF_MATCH_PREFIX) ? "(*)" : "");
 			if (l > end - p)
 				continue;
 			p += l;
@@ -75,7 +78,7 @@ static inline void action_build_list(struct list *keywords,
 }
 
 /* Check an action ruleset validity. It returns the number of error encountered
- * andd err_code is updated if a warning is emitted.
+ * and err_code is updated if a warning is emitted.
  */
 int check_action_rules(struct list *rules, struct proxy *px, int *err_code);
 
@@ -95,4 +98,16 @@ int check_trk_action(struct act_rule *rule, struct proxy *px, char **err);
  */
 int check_capture(struct act_rule *rule, struct proxy *px, char **err);
 
+int cfg_parse_rule_set_timeout(const char **args, int idx, int *out_timeout,
+                               enum act_timeout_name *name,
+                               struct sample_expr **expr, char **err,
+                               const char *file, int line, struct arg_list *al);
+
+static inline void release_timeout_action(struct act_rule *rule)
+{
+	release_sample_expr(rule->arg.timeout.expr);
+}
+
+void free_act_rules(struct list *rules);
+
 #endif /* _HAPROXY_ACTION_H */
