@@ -23,8 +23,11 @@
 #define _HAPROXY_CFGPARSE_H
 
 #include <haproxy/api.h>
-#include <haproxy/errors.h>
-#include <haproxy/proxy.h>
+
+struct hap_cpuset;
+struct proxy;
+struct bind_conf;
+struct acl_cond;
 
 /* configuration sections */
 #define CFG_NONE	0
@@ -40,6 +43,11 @@ enum kw_mod {
 	KWM_DEF,      /* "default" prefixed before the keyword */
 };
 
+enum cfg_keyword_flags {
+	KWF_EXPERIMENTAL = 0x1,
+	KWF_MATCH_PREFIX = 0x2,
+};
+
 struct cfg_keyword {
 	int section;                            /* section type for this keyword */
 	const char *kw;                         /* the keyword itself */
@@ -47,10 +55,11 @@ struct cfg_keyword {
 		     char **args,               /* command line and arguments */
 		     int section_type,          /* current section CFG_{GLOBAL|LISTEN} */
 		     struct proxy *curpx,       /* current proxy (NULL in GLOBAL) */
-		     struct proxy *defpx,       /* default proxy (NULL in GLOBAL) */
+		     const struct proxy *defpx, /* default proxy (NULL in GLOBAL) */
 		     const char *file,          /* config file name */
 		     int line,                  /* config file line number */
 		     char **err);               /* error or warning message output pointer */
+	int flags;
 };
 
 /* A keyword list. It is a NULL-terminated array of keywords. It embeds a
@@ -84,7 +93,6 @@ extern int cfg_maxconn;
 extern char *cfg_scope;
 extern struct cfg_kw_list cfg_keywords;
 extern char *cursection;
-extern struct proxy defproxy;
 
 int cfg_parse_global(const char *file, int linenum, char **args, int inv);
 int cfg_parse_listen(const char *file, int linenum, char **args, int inv);
@@ -93,7 +101,6 @@ int cfg_parse_track_sc_num(unsigned int *track_sc_num,
 int readcfgfile(const char *file);
 void cfg_register_keywords(struct cfg_kw_list *kwl);
 void cfg_unregister_keywords(struct cfg_kw_list *kwl);
-void init_default_instance();
 int check_config_validity();
 int str2listener(char *str, struct proxy *curproxy, struct bind_conf *bind_conf, const char *file, int line, char **err);
 int str2receiver(char *str, struct proxy *curproxy, struct bind_conf *bind_conf, const char *file, int line, char **err);
@@ -108,63 +115,17 @@ int warnif_misplaced_tcp_conn(struct proxy *proxy, const char *file, int line, c
 int warnif_misplaced_tcp_sess(struct proxy *proxy, const char *file, int line, const char *arg);
 int warnif_misplaced_tcp_cont(struct proxy *proxy, const char *file, int line, const char *arg);
 int warnif_cond_conflicts(const struct acl_cond *cond, unsigned int where, const char *file, int line);
+int warnif_tcp_http_cond(const struct proxy *px, const struct acl_cond *cond);
 int too_many_args_idx(int maxarg, int index, char **args, char **msg, int *err_code);
 int too_many_args(int maxarg, char **args, char **msg, int *err_code);
 int alertif_too_many_args_idx(int maxarg, int index, const char *file, int linenum, char **args, int *err_code);
 int alertif_too_many_args(int maxarg, const char *file, int linenum, char **args, int *err_code);
 int parse_process_number(const char *arg, unsigned long *proc, int max, int *autoinc, char **err);
-unsigned long parse_cpu_set(const char **args, unsigned long *cpu_set, char **err);
+unsigned long parse_cpu_set(const char **args, struct hap_cpuset *cpu_set, int comma_allowed, char **err);
 void free_email_alert(struct proxy *p);
-
-/*
- * Sends a warning if proxy <proxy> does not have at least one of the
- * capabilities in <cap>. An optional <hint> may be added at the end
- * of the warning to help the user. Returns 1 if a warning was emitted
- * or 0 if the condition is valid.
- */
-static inline int warnifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint)
-{
-	char *msg;
-
-	switch (cap) {
-	case PR_CAP_BE: msg = "no backend"; break;
-	case PR_CAP_FE: msg = "no frontend"; break;
-	case PR_CAP_BE|PR_CAP_FE: msg = "neither frontend nor backend"; break;
-	default: msg = "not enough"; break;
-	}
-
-	if (!(proxy->cap & cap)) {
-		ha_warning("parsing [%s:%d] : '%s' ignored because %s '%s' has %s capability.%s\n",
-			   file, line, arg, proxy_type_str(proxy), proxy->id, msg, hint ? hint : "");
-		return 1;
-	}
-	return 0;
-}
-
-/*
- * Sends an alert if proxy <proxy> does not have at least one of the
- * capabilities in <cap>. An optional <hint> may be added at the end
- * of the alert to help the user. Returns 1 if an alert was emitted
- * or 0 if the condition is valid.
- */
-static inline int failifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint)
-{
-	char *msg;
-
-	switch (cap) {
-	case PR_CAP_BE: msg = "no backend"; break;
-	case PR_CAP_FE: msg = "no frontend"; break;
-	case PR_CAP_BE|PR_CAP_FE: msg = "neither frontend nor backend"; break;
-	default: msg = "not enough"; break;
-	}
-
-	if (!(proxy->cap & cap)) {
-		ha_alert("parsing [%s:%d] : '%s' not allowed because %s '%s' has %s capability.%s\n",
-			 file, line, arg, proxy_type_str(proxy), proxy->id, msg, hint ? hint : "");
-		return 1;
-	}
-	return 0;
-}
+const char *cfg_find_best_match(const char *word, const struct list *list, int section, const char **extra);
+int warnifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint);
+int failifnotcap(struct proxy *proxy, int cap, const char *file, int line, const char *arg, const char *hint);
 
 /* simplified way to define a section parser */
 #define REGISTER_CONFIG_SECTION(name, parse, post)                            \
