@@ -56,7 +56,10 @@ enum {
 	STKTABLE_DT_BYTES_OUT_RATE,/* bytes rate from servers to client */
 	STKTABLE_DT_GPC1,         /* General Purpose Counter 1 (unsigned 32-bit integer) */
 	STKTABLE_DT_GPC1_RATE,    /* General Purpose Counter 1's event rate */
-	STKTABLE_DT_SERVER_NAME,  /* The server name */
+	STKTABLE_DT_SERVER_KEY,   /* The server key */
+	STKTABLE_DT_HTTP_FAIL_CNT, /* cumulated number of HTTP server failures */
+	STKTABLE_DT_HTTP_FAIL_RATE,/* HTTP server failures rate */
+
 	STKTABLE_STATIC_DATA_TYPES,/* number of types above */
 	/* up to STKTABLE_EXTRA_DATA_TYPES types may be registered here, always
 	 * followed by the number of data types, must always be last.
@@ -69,7 +72,7 @@ enum {
 	STD_T_SINT = 0,           /* data is of type signed int */
 	STD_T_UINT,               /* data is of type unsigned int */
 	STD_T_ULL,                /* data is of type unsigned long long */
-	STD_T_FRQP,               /* data is of type freq_ctr_period */
+	STD_T_FRQP,               /* data is of type freq_ctr */
 	STD_T_DICT,               /* data is of type key of dictionary entry */
 };
 
@@ -80,6 +83,12 @@ enum {
 	ARG_T_DELAY,              /* a delay which supports time units */
 };
 
+/* They types of keys that servers can be identified by */
+enum {
+	STKTABLE_SRV_NAME = 0,
+	STKTABLE_SRV_ADDR,
+};
+
 /* stick table key type flags */
 #define STK_F_CUSTOM_KEYSIZE      0x00000001   /* this table's key size is configurable */
 
@@ -107,30 +116,32 @@ union stktable_data {
 	int std_t_sint;
 	unsigned int std_t_uint;
 	unsigned long long std_t_ull;
-	struct freq_ctr_period std_t_frqp;
+	struct freq_ctr std_t_frqp;
 	struct dict_entry *std_t_dict;
 
 	/* types of each storable data */
 	int server_id;
-	struct dict_entry *server_name;
+	struct dict_entry *server_key;
 	unsigned int gpt0;
 	unsigned int gpc0;
-	struct freq_ctr_period gpc0_rate;
+	struct freq_ctr gpc0_rate;
 	unsigned int gpc1;
-	struct freq_ctr_period gpc1_rate;
+	struct freq_ctr gpc1_rate;
 	unsigned int conn_cnt;
-	struct freq_ctr_period conn_rate;
+	struct freq_ctr conn_rate;
 	unsigned int conn_cur;
 	unsigned int sess_cnt;
-	struct freq_ctr_period sess_rate;
+	struct freq_ctr sess_rate;
 	unsigned int http_req_cnt;
-	struct freq_ctr_period http_req_rate;
+	struct freq_ctr http_req_rate;
 	unsigned int http_err_cnt;
-	struct freq_ctr_period http_err_rate;
+	struct freq_ctr http_err_rate;
 	unsigned long long bytes_in_cnt;
-	struct freq_ctr_period bytes_in_rate;
+	struct freq_ctr bytes_in_rate;
 	unsigned long long bytes_out_cnt;
-	struct freq_ctr_period bytes_out_rate;
+	struct freq_ctr bytes_out_rate;
+	unsigned int http_fail_cnt;
+	struct freq_ctr http_fail_rate;
 };
 
 /* known data types */
@@ -189,6 +200,7 @@ struct stktable {
 	} peers;
 
 	unsigned long type;       /* type of table (determines key format) */
+	unsigned int server_key_type; /* What type of key is used to identify servers */
 	size_t key_size;          /* size of a key, maximum size in case of string */
 	unsigned int size;        /* maximum number of sticky sessions in table */
 	unsigned int current;     /* number of sticky sessions currently in table */
