@@ -239,7 +239,7 @@ void *malloc(size_t size)
 		return memprof_malloc_handler(size);
 
 	ret = memprof_malloc_handler(size);
-	size = malloc_usable_size(ret);
+	size = malloc_usable_size(ret) + sizeof(void *);
 
 	bin = memprof_get_bin(__builtin_return_address(0), MEMPROF_METH_MALLOC);
 	_HA_ATOMIC_ADD(&bin->alloc_calls, 1);
@@ -264,7 +264,7 @@ void *calloc(size_t nmemb, size_t size)
 		return memprof_calloc_handler(nmemb, size);
 
 	ret = memprof_calloc_handler(nmemb, size);
-	size = malloc_usable_size(ret);
+	size = malloc_usable_size(ret) + sizeof(void *);
 
 	bin = memprof_get_bin(__builtin_return_address(0), MEMPROF_METH_CALLOC);
 	_HA_ATOMIC_ADD(&bin->alloc_calls, 1);
@@ -295,6 +295,10 @@ void *realloc(void *ptr, size_t size)
 	ret = memprof_realloc_handler(ptr, size);
 	size = malloc_usable_size(ret);
 
+	/* only count the extra link for new allocations */
+	if (!ptr)
+		size += sizeof(void *);
+
 	bin = memprof_get_bin(__builtin_return_address(0), MEMPROF_METH_REALLOC);
 	if (size > size_before) {
 		_HA_ATOMIC_ADD(&bin->alloc_calls, 1);
@@ -327,7 +331,7 @@ void free(void *ptr)
 		return;
 	}
 
-	size_before = malloc_usable_size(ptr);
+	size_before = malloc_usable_size(ptr) + sizeof(void *);
 	memprof_free_handler(ptr);
 
 	bin = memprof_get_bin(__builtin_return_address(0), MEMPROF_METH_FREE);
@@ -675,9 +679,16 @@ static int cli_io_handler_show_profiling(struct appctx *appctx)
 		else
 			chunk_appendf(&trash, "[other]");
 
-		chunk_appendf(&trash," %s(%lld)\n", memprof_methods[entry->method],
+		chunk_appendf(&trash," %s(%lld)", memprof_methods[entry->method],
 			      (long long)(entry->alloc_tot - entry->free_tot) / (long long)(entry->alloc_calls + entry->free_calls));
 
+		if (entry->alloc_tot && entry->free_tot) {
+			/* that's a realloc, show the total diff to help spot leaks */
+			chunk_appendf(&trash," [delta=%lld]", (long long)(entry->alloc_tot - entry->free_tot));
+		}
+
+		chunk_appendf(&trash, "\n");
+
 		if (ci_putchk(si_ic(si), &trash) == -1) {
 			si_rx_room_blk(si);
 			return 0;
