@@ -26,7 +26,7 @@
 
 /* private data */
 static THREAD_LOCAL struct epoll_event *epoll_events = NULL;
-static int epoll_fd[MAX_THREADS]; // per-thread epoll_fd
+static int epoll_fd[MAX_THREADS] __read_mostly; // per-thread epoll_fd
 
 #ifndef EPOLLRDHUP
 /* EPOLLRDHUP was defined late in libc, and it appeared in kernel 2.6.17 */
@@ -41,7 +41,7 @@ static int epoll_fd[MAX_THREADS]; // per-thread epoll_fd
  */
 static void __fd_clo(int fd)
 {
-	if (unlikely(fdtab[fd].cloned)) {
+	if (unlikely(fdtab[fd].state & FD_CLONED)) {
 		unsigned long m = polled_mask[fd].poll_recv | polled_mask[fd].poll_send;
 		struct epoll_event ev;
 		int i;
@@ -60,7 +60,7 @@ static void _update_fd(int fd)
 	en = fdtab[fd].state;
 
 	/* Try to force EPOLLET on FDs that support it */
-	if (fdtab[fd].et_possible) {
+	if (fdtab[fd].state & FD_ET_POSSIBLE) {
 		/* already done ? */
 		if (polled_mask[fd].poll_recv & polled_mask[fd].poll_send & tid_bit)
 			return;
@@ -185,7 +185,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 
 	thread_harmless_now();
 
-	/* now let's wait for polled events */
+	/* Now let's wait for polled events. */
 	wait_time = wake ? 0 : compute_poll_timeout(exp);
 	tv_entering_poll();
 	activity_count_runtime();
@@ -201,7 +201,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		}
 		if (timeout || !wait_time)
 			break;
-		if (signal_queue_len || wake)
+		if (wake)
 			break;
 		if (tick_isset(exp) && tick_is_expired(exp, now_ms))
 			break;
@@ -223,7 +223,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		fd = epoll_events[count].data.fd;
 
 #ifdef DEBUG_FD
-		_HA_ATOMIC_ADD(&fdtab[fd].event_count, 1);
+		_HA_ATOMIC_INC(&fdtab[fd].event_count);
 #endif
 		if (!fdtab[fd].owner) {
 			activity[tid].poll_dead_fd++;
@@ -287,8 +287,7 @@ static void deinit_epoll_per_thread()
 	if (MAX_THREADS > 1 && tid)
 		close(epoll_fd[tid]);
 
-	free(epoll_events);
-	epoll_events = NULL;
+	ha_free(&epoll_events);
 }
 
 /*
