@@ -292,10 +292,9 @@ flt_init_all()
 	int err_code = ERR_NONE;
 
 	for (px = proxies_list; px; px = px->next) {
-		if (px->disabled) {
-			flt_deinit(px);
+		if (px->disabled)
 			continue;
-		}
+
 		err_code |= flt_init(px);
 		if (err_code & (ERR_ABORT|ERR_FATAL)) {
 			ha_alert("Failed to initialize filters for proxy '%s'.\n",
@@ -473,8 +472,10 @@ flt_stream_start(struct stream *s)
 		if (FLT_OPS(filter)->stream_start && FLT_OPS(filter)->stream_start(s, filter) < 0)
 			return -1;
 	}
-	if (strm_li(s) && (strm_li(s)->analysers & AN_REQ_FLT_START_FE))
+	if (strm_li(s) && (strm_li(s)->analysers & AN_REQ_FLT_START_FE)) {
 		s->req.flags |= CF_FLT_ANALYZE;
+		s->req.analysers |= AN_REQ_FLT_END;
+	}
 	return 0;
 }
 
@@ -533,10 +534,14 @@ flt_set_stream_backend(struct stream *s, struct proxy *be)
 		    FLT_OPS(filter)->stream_set_backend(s, filter, be) < 0)
 			return -1;
 	}
-	if (be->be_req_ana & AN_REQ_FLT_START_BE)
+	if (be->be_req_ana & AN_REQ_FLT_START_BE) {
 		s->req.flags |= CF_FLT_ANALYZE;
-	if ((strm_fe(s)->fe_rsp_ana | be->be_rsp_ana) & (AN_RES_FLT_START_FE|AN_RES_FLT_START_BE))
+		s->req.analysers |= AN_REQ_FLT_END;
+	}
+	if ((strm_fe(s)->fe_rsp_ana | be->be_rsp_ana) & (AN_RES_FLT_START_FE|AN_RES_FLT_START_BE)) {
 		s->res.flags |= CF_FLT_ANALYZE;
+		s->res.analysers |= AN_RES_FLT_END;
+	}
 
 	return 0;
 }
@@ -706,6 +711,7 @@ flt_start_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 
 	/* Set flag on channel to tell that the channel is filtered */
 	chn->flags |= CF_FLT_ANALYZE;
+	chn->analysers |= ((chn->flags & CF_ISRESP) ? AN_RES_FLT_END : AN_REQ_FLT_END);
 
 	RESUME_FILTER_LOOP(s, chn) {
 		if (!(chn->flags & CF_ISRESP)) {
@@ -757,6 +763,7 @@ flt_pre_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 			ret = FLT_OPS(filter)->channel_pre_analyze(s, filter, chn, an_bit);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, check_result);
+			filter->pre_analyzers &= ~an_bit;
 		}
 	} RESUME_FILTER_END;
 
@@ -789,6 +796,7 @@ flt_post_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 			ret = FLT_OPS(filter)->channel_post_analyze(s, filter, chn, an_bit);
 			if (ret < 0)
 				break;
+			filter->post_analyzers &= ~an_bit;
 		}
 	}
 	ret = handle_analyzer_result(s, chn, 0, ret);
