@@ -33,6 +33,7 @@
 #include <haproxy/stats.h>
 #include <haproxy/stick_table.h>
 #include <haproxy/time.h>
+#include <haproxy/tools.h>
 
 /* Contains the class reference of the concat object. */
 static int class_concat_ref;
@@ -355,7 +356,7 @@ static int hlua_get_info(lua_State *L)
 {
 	int i;
 
-	stats_fill_info(stats, STATS_LEN);
+	stats_fill_info(stats, STATS_LEN, 0);
 
 	lua_newtable(L);
 	for (i=0; i<INF_TOTAL_FIELDS; i++) {
@@ -867,7 +868,8 @@ int hlua_listener_get_stats(lua_State *L)
 		return 1;
 	}
 
-	stats_fill_li_stats(li->bind_conf->frontend, li, STAT_SHLGNDS, stats, STATS_LEN);
+	stats_fill_li_stats(li->bind_conf->frontend, li, STAT_SHLGNDS, stats,
+			    STATS_LEN, NULL);
 
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
@@ -923,7 +925,8 @@ int hlua_server_get_stats(lua_State *L)
 		return 1;
 	}
 
-	stats_fill_sv_stats(srv->proxy, srv, STAT_SHLGNDS, stats, STATS_LEN);
+	stats_fill_sv_stats(srv->proxy, srv, STAT_SHLGNDS, stats,
+			    STATS_LEN, NULL);
 
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
@@ -1054,7 +1057,7 @@ int hlua_server_set_addr(lua_State *L)
 		port = NULL;
 
 	HA_SPIN_LOCK(SERVER_LOCK, &srv->lock);
-	err = update_server_addr_port(srv, addr, port, "Lua script");
+	err = srv_update_addr_port(srv, addr, port, "Lua script");
 	HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 	if (!err)
 		lua_pushnil(L);
@@ -1302,6 +1305,7 @@ int hlua_proxy_pause(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - pause_proxy takes it */
 	pause_proxy(px);
 	return 0;
 }
@@ -1311,6 +1315,7 @@ int hlua_proxy_resume(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - resume_proxy takes it */
 	resume_proxy(px);
 	return 0;
 }
@@ -1320,6 +1325,7 @@ int hlua_proxy_stop(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
+	/* safe to call without PROXY_LOCK - stop_proxy takes it */
 	stop_proxy(px);
 	return 0;
 }
@@ -1342,9 +1348,9 @@ int hlua_proxy_get_stats(lua_State *L)
 
 	px = hlua_check_proxy(L, 1);
 	if (px->cap & PR_CAP_BE)
-		stats_fill_be_stats(px, STAT_SHLGNDS, stats, STATS_LEN);
+		stats_fill_be_stats(px, STAT_SHLGNDS, stats, STATS_LEN, NULL);
 	else
-		stats_fill_fe_stats(px, stats, STATS_LEN);
+		stats_fill_fe_stats(px, stats, STATS_LEN, NULL);
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
 		lua_pushstring(L, stat_fields[i].name);
@@ -1480,24 +1486,24 @@ int hlua_tokenize(lua_State *L)
 
 int hlua_parse_addr(lua_State *L)
 {
-	struct hlua_addr *addr;
+	struct net_addr *addr;
 	const char *str = luaL_checkstring(L, 1);
 	unsigned char mask;
 
-	addr = lua_newuserdata(L, sizeof(struct hlua_addr));
+	addr = lua_newuserdata(L, sizeof(struct net_addr));
 	if (!addr) {
 		lua_pushnil(L);
 		return 1;
 	}
 
 	if (str2net(str, PAT_MF_NO_DNS, &addr->addr.v4.ip, &addr->addr.v4.mask)) {
-		addr->type = AF_INET;
+		addr->family = AF_INET;
 		return 1;
 	}
 
 	if (str62net(str, &addr->addr.v6.ip, &mask)) {
 		len2mask6(mask, &addr->addr.v6.mask);
-		addr->type = AF_INET6;
+		addr->family = AF_INET6;
 		return 1;
 	}
 
@@ -1508,8 +1514,8 @@ int hlua_parse_addr(lua_State *L)
 
 int hlua_match_addr(lua_State *L)
 {
-	struct hlua_addr *addr1;
-	struct hlua_addr *addr2;
+	struct net_addr *addr1;
+	struct net_addr *addr2;
 
 	if (!lua_isuserdata(L, 1) ||
 	    !lua_isuserdata(L, 2)) {
@@ -1520,12 +1526,12 @@ int hlua_match_addr(lua_State *L)
 	addr1 = lua_touserdata(L, 1);
 	addr2 = lua_touserdata(L, 2);
 
-	if (addr1->type != addr2->type) {
+	if (addr1->family != addr2->family) {
 		lua_pushboolean(L, 0);
 		return 1;
 	}
 
-	if (addr1->type == AF_INET) {
+	if (addr1->family == AF_INET) {
 		if ((addr1->addr.v4.ip.s_addr & addr2->addr.v4.mask.s_addr) ==
 		    (addr2->addr.v4.ip.s_addr & addr1->addr.v4.mask.s_addr)) {
 			lua_pushboolean(L, 1);
