@@ -19,6 +19,7 @@
 #include <import/eb32tree.h>
 #include <haproxy/api.h>
 #include <haproxy/backend.h>
+#include <haproxy/errors.h>
 #include <haproxy/queue.h>
 #include <haproxy/server-t.h>
 #include <haproxy/tools.h>
@@ -124,7 +125,7 @@ static void chash_set_server_status_down(struct server *srv)
 	if (!srv_lb_status_changed(srv))
                return;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	if (srv_willbe_usable(srv))
 		goto out_update_state;
@@ -162,7 +163,7 @@ out_update_backend:
  out_update_state:
 	srv_lb_commit_status(srv);
 
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 }
 
 /* This function updates the server trees according to server <srv>'s new
@@ -181,7 +182,7 @@ static void chash_set_server_status_up(struct server *srv)
 	if (!srv_lb_status_changed(srv))
                return;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	if (!srv_willbe_usable(srv))
 		goto out_update_state;
@@ -224,7 +225,7 @@ static void chash_set_server_status_up(struct server *srv)
  out_update_state:
 	srv_lb_commit_status(srv);
 
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 }
 
 /* This function must be called after an update to server <srv>'s effective
@@ -264,7 +265,7 @@ static void chash_update_server_weight(struct server *srv)
 		return;
 	}
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	/* only adjust the server's presence in the tree */
 	chash_queue_dequeue_srv(srv);
@@ -277,7 +278,7 @@ static void chash_update_server_weight(struct server *srv)
 	update_backend_weight(p);
 	srv_lb_commit_status(srv);
 
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 }
 
 /*
@@ -324,7 +325,7 @@ struct server *chash_get_server_hash(struct proxy *p, unsigned int hash, const s
 	unsigned int dn, dp;
 	int loop;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_RDLOCK(LBPRM_LOCK, &p->lbprm.lock);
 
 	if (p->srv_act)
 		root = &p->lbprm.chash.act;
@@ -379,7 +380,7 @@ struct server *chash_get_server_hash(struct proxy *p, unsigned int hash, const s
 	}
 
  out:
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_RDUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 	return nsrv;
 }
 
@@ -395,7 +396,7 @@ struct server *chash_get_next_server(struct proxy *p, struct server *srvtoavoid)
 	srv = avoided = NULL;
 	avoided_node = NULL;
 
-	HA_SPIN_LOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRLOCK(LBPRM_LOCK, &p->lbprm.lock);
 	if (p->srv_act)
 		root = &p->lbprm.chash.act;
 	else if (p->lbprm.fbck) {
@@ -454,7 +455,7 @@ struct server *chash_get_next_server(struct proxy *p, struct server *srvtoavoid)
 	}
 
  out:
-	HA_SPIN_UNLOCK(LBPRM_LOCK, &p->lbprm.lock);
+	HA_RWLOCK_WRUNLOCK(LBPRM_LOCK, &p->lbprm.lock);
 	return srv;
 }
 
@@ -462,8 +463,9 @@ struct server *chash_get_next_server(struct proxy *p, struct server *srvtoavoid)
  * constistent hashing. The servers receive an array of initialized nodes
  * with their assigned keys. It also sets p->lbprm.wdiv to the eweight to
  * uweight ratio.
+ * Return 0 in case of success, -1 in case of allocation failure.
  */
-void chash_init_server_tree(struct proxy *p)
+int chash_init_server_tree(struct proxy *p)
 {
 	struct server *srv;
 	struct eb_root init_head = EB_ROOT;
@@ -493,7 +495,12 @@ void chash_init_server_tree(struct proxy *p)
 		srv->lb_tree = (srv->flags & SRV_F_BACKUP) ? &p->lbprm.chash.bck : &p->lbprm.chash.act;
 		srv->lb_nodes_tot = srv->uweight * BE_WEIGHT_SCALE;
 		srv->lb_nodes_now = 0;
-		srv->lb_nodes = calloc(srv->lb_nodes_tot, sizeof(struct tree_occ));
+		srv->lb_nodes = calloc(srv->lb_nodes_tot,
+				       sizeof(*srv->lb_nodes));
+		if (!srv->lb_nodes) {
+			ha_alert("failed to allocate lb_nodes for server %s.\n", srv->id);
+			return -1;
+		}
 		for (node = 0; node < srv->lb_nodes_tot; node++) {
 			srv->lb_nodes[node].server = srv;
 			srv->lb_nodes[node].node.key = full_hash(srv->puid * SRV_EWGHT_RANGE + node);
@@ -502,4 +509,5 @@ void chash_init_server_tree(struct proxy *p)
 		if (srv_currently_usable(srv))
 			chash_queue_dequeue_srv(srv);
 	}
+	return 0;
 }
