@@ -11,6 +11,7 @@
  */
 
 #include <stdio.h>
+#include <syslog.h>
 
 #include <haproxy/api.h>
 #include <haproxy/applet-t.h>
@@ -120,7 +121,6 @@ int sample_load_map(struct arg *arg, struct sample_conv *conv,
 	desc->pat.match = pat_match_fcts[(long)conv->private];
 	desc->pat.parse = pat_parse_fcts[(long)conv->private];
 	desc->pat.index = pat_index_fcts[(long)conv->private];
-	desc->pat.delete = pat_delete_fcts[(long)conv->private];
 	desc->pat.prune = pat_prune_fcts[(long)conv->private];
 	desc->pat.expect_type = pat_match_types[(long)conv->private];
 
@@ -320,6 +320,7 @@ struct pattern_expr *pat_expr_get_next(struct pattern_expr *getnext, struct list
 	return expr;
 }
 
+/* expects the current generation ID in appctx->cli.cli.i0 */
 static int cli_io_handler_pat_list(struct appctx *appctx)
 {
 	struct stream_interface *si = appctx->owner;
@@ -330,10 +331,12 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 		 * reference to the last ref_elt being dumped.
 		 */
 		if (appctx->st2 == STAT_ST_LIST) {
+			HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			if (!LIST_ISEMPTY(&appctx->ctx.map.bref.users)) {
-				LIST_DEL(&appctx->ctx.map.bref.users);
+				LIST_DELETE(&appctx->ctx.map.bref.users);
 				LIST_INIT(&appctx->ctx.map.bref.users);
 			}
+			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		}
 		return 1;
 	}
@@ -341,27 +344,17 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 	switch (appctx->st2) {
 
 	case STAT_ST_INIT:
-		/* the function had not been called yet, let's prepare the
-		 * buffer for a response. We initialize the current stream
-		 * pointer to the first in the global list. When a target
-		 * stream is being destroyed, it is responsible for updating
-		 * this pointer. We know we have reached the end when this
-		 * pointer points back to the head of the streams list.
-		 */
-		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-		LIST_INIT(&appctx->ctx.map.bref.users);
-		appctx->ctx.map.bref.ref = appctx->ctx.map.ref->head.n;
-		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		appctx->st2 = STAT_ST_LIST;
 		/* fall through */
 
 	case STAT_ST_LIST:
-
 		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 
 		if (!LIST_ISEMPTY(&appctx->ctx.map.bref.users)) {
-			LIST_DEL(&appctx->ctx.map.bref.users);
+			LIST_DELETE(&appctx->ctx.map.bref.users);
 			LIST_INIT(&appctx->ctx.map.bref.users);
+		} else {
+			appctx->ctx.map.bref.ref = appctx->ctx.map.ref->head.n;
 		}
 
 		while (appctx->ctx.map.bref.ref != &appctx->ctx.map.ref->head) {
@@ -369,6 +362,9 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 
 			elt = LIST_ELEM(appctx->ctx.map.bref.ref, struct pat_ref_elt *, list);
 
+			if (elt->gen_id != appctx->ctx.cli.i0)
+				goto skip;
+
 			/* build messages */
 			if (elt->sample)
 				chunk_appendf(&trash, "%p %s %s\n",
@@ -382,12 +378,12 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 				/* let's try again later from this stream. We add ourselves into
 				 * this stream's users so that it can remove us upon termination.
 				 */
-				LIST_ADDQ(&elt->back_refs, &appctx->ctx.map.bref.users);
+				LIST_APPEND(&elt->back_refs, &appctx->ctx.map.bref.users);
 				HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 				si_rx_room_blk(si);
 				return 0;
 			}
-
+		skip:
 			/* get next list entry and check the end of the list */
 			appctx->ctx.map.bref.ref = elt->list.n;
 		}
@@ -435,9 +431,9 @@ static int cli_io_handler_pats_list(struct appctx *appctx)
 			/* Build messages. If the reference is used by another category than
 			 * the listed categories, display the information in the message.
 			 */
-			chunk_appendf(&trash, "%d (%s) %s\n", appctx->ctx.map.ref->unique_id,
+			chunk_appendf(&trash, "%d (%s) %s. curr_ver=%u next_ver=%u\n", appctx->ctx.map.ref->unique_id,
 			              appctx->ctx.map.ref->reference ? appctx->ctx.map.ref->reference : "",
-			              appctx->ctx.map.ref->display);
+			              appctx->ctx.map.ref->display, appctx->ctx.map.ref->curr_gen, appctx->ctx.map.ref->next_gen);
 
 			if (ci_putchk(si_ic(si), &trash) == -1) {
 				/* let's try again later from this stream. We add ourselves into
@@ -583,8 +579,7 @@ static int cli_io_handler_map_lookup(struct appctx *appctx)
 
 static void cli_release_mlook(struct appctx *appctx)
 {
-	free(appctx->ctx.map.chunk.area);
-	appctx->ctx.map.chunk.area = NULL;
+	ha_free(&appctx->ctx.map.chunk.area);
 }
 
 
@@ -629,12 +624,41 @@ static int cli_parse_get_map(char **args, char *payload, struct appctx *appctx,
 	return 1;
 }
 
+static int cli_parse_prepare_map(char **args, char *payload, struct appctx *appctx, void *private)
+{
+	if (strcmp(args[1], "map") == 0 ||
+	    strcmp(args[1], "acl") == 0) {
+		uint next_gen;
+		char *msg = NULL;
+
+		/* Set ACL or MAP flags. */
+		if (args[1][0] == 'm')
+			appctx->ctx.map.display_flags = PAT_REF_MAP;
+		else
+			appctx->ctx.map.display_flags = PAT_REF_ACL;
+
+		/* lookup into the refs and check the map flag */
+		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
+		if (!appctx->ctx.map.ref ||
+		    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags)) {
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
+		}
+		next_gen = pat_ref_newgen(appctx->ctx.map.ref);
+		return cli_dynmsg(appctx, LOG_INFO, memprintf(&msg, "New version created: %u\n", next_gen));
+	}
+
+	return 0;
+}
+
 static void cli_release_show_map(struct appctx *appctx)
 {
 	if (appctx->st2 == STAT_ST_LIST) {
 		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		if (!LIST_ISEMPTY(&appctx->ctx.map.bref.users))
-			LIST_DEL(&appctx->ctx.map.bref.users);
+			LIST_DELETE(&appctx->ctx.map.bref.users);
 		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 	}
 }
@@ -643,6 +667,7 @@ static int cli_parse_show_map(char **args, char *payload, struct appctx *appctx,
 {
 	if (strcmp(args[1], "map") == 0 ||
 	    strcmp(args[1], "acl") == 0) {
+		const char *gen = NULL;
 
 		/* Set ACL or MAP flags. */
 		if (args[1][0] == 'm')
@@ -656,6 +681,15 @@ static int cli_parse_show_map(char **args, char *payload, struct appctx *appctx,
 			return 0;
 		}
 
+		/* For both "map" and "acl" we may have an optional generation
+		 * number specified using a "@" character before the pattern
+		 * file name.
+		 */
+		if (*args[2] == '@') {
+			gen = args[2] + 1;
+			args++;
+		}
+
 		/* lookup into the refs and check the map flag */
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 		if (!appctx->ctx.map.ref ||
@@ -665,6 +699,14 @@ static int cli_parse_show_map(char **args, char *payload, struct appctx *appctx,
 			else
 				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
+
+		/* set the desired generation id in cli.i0 */
+		if (gen)
+			appctx->ctx.cli.i0 = str2uic(gen);
+		else
+			appctx->ctx.cli.i0 = appctx->ctx.map.ref->curr_gen;
+
+		LIST_INIT(&appctx->ctx.map.bref.users);
 		appctx->io_handler = cli_io_handler_pat_list;
 		appctx->io_release = cli_release_show_map;
 		return 0;
@@ -743,24 +785,12 @@ static int cli_parse_set_map(char **args, char *payload, struct appctx *appctx,
 	return 1;
 }
 
-static int map_add_key_value(struct appctx *appctx, const char *key, const char *value, char **err)
-{
-	int ret;
-
-	HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-	if (appctx->ctx.map.display_flags == PAT_REF_MAP)
-		ret = pat_ref_add(appctx->ctx.map.ref, key, value, err);
-	else
-		ret = pat_ref_add(appctx->ctx.map.ref, key, NULL, err);
-	HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-
-	return ret;
-}
-
 static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx, void *private)
 {
 	if (strcmp(args[1], "map") == 0 ||
 	    strcmp(args[1], "acl") == 0) {
+		const char *gen = NULL;
+		uint genid = 0;
 		int ret;
 		char *err;
 
@@ -770,6 +800,15 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 		else
 			appctx->ctx.map.display_flags = PAT_REF_ACL;
 
+		/* For both "map" and "acl" we may have an optional generation
+		 * number specified using a "@" character before the pattern
+		 * file name.
+		 */
+		if (*args[2] == '@') {
+			gen = args[2] + 1;
+			args++;
+		}
+
 		/* If the keyword is "map", we expect:
 		 *   - three parameters if there is no payload
 		 *   - one parameter if there is a payload
@@ -794,6 +833,16 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
 
+		if (gen) {
+			genid = str2uic(gen);
+			if ((int)(genid - appctx->ctx.map.ref->next_gen) > 0) {
+				if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+					return cli_err(appctx, "Version number in the future, please use 'prepare map' before.\n");
+				else
+					return cli_err(appctx, "Version number in the future, please use 'prepare acl' before.\n");
+			}
+		}
+
 		/* The command "add acl" is prohibited if the reference
 		 * use samples.
 		 */
@@ -803,25 +852,24 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 				       "This ACL is shared with a map containing samples. "
 				       "You must use the command 'add map' to add values.\n");
 		}
-		/* Add value(s). */
+
+		/* Add value(s). If no payload is used, key and value are read
+		 * from the command line and only one key is set. If a payload
+		 * is passed, one key/value pair is read per line till the end
+		 * of the payload is reached.
+		 */
 		err = NULL;
-		if (!payload) {
-			ret = map_add_key_value(appctx, args[3], args[4], &err);
-			if (!ret) {
-				if (err)
-					return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
-				else
-					return cli_err(appctx, "Failed to add an entry.\n");
-			}
-		}
-		else {
-			const char *end = payload + strlen(payload);
 
-			while (payload < end) {
-				char *key, *value;
-				size_t l;
+		do {
+			char *key   = args[3];
+			char *value = args[4];
+			size_t l;
+
+			if (payload) {
+				/* key and value passed as payload, one pair per line */
+				if (!*payload)
+					break;
 
-				/* key */
 				key = payload;
 				l = strcspn(key, " \t");
 				payload += l;
@@ -840,16 +888,22 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 				if (*payload)
 					payload++;
 				value[l] = 0;
-
-				ret = map_add_key_value(appctx, key, value, &err);
-				if (!ret) {
-					if (err)
-						return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
-					else
-						return cli_err(appctx, "Failed to add a key.\n");
-				}
 			}
-		}
+
+			if (appctx->ctx.map.display_flags != PAT_REF_MAP)
+				value = NULL;
+
+			HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+			ret = !!pat_ref_load(appctx->ctx.map.ref, gen ? genid : appctx->ctx.map.ref->curr_gen, key, value, -1, &err);
+			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+
+			if (!ret) {
+				if (err)
+					return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
+				else
+					return cli_err(appctx, "Failed to add a key.\n");
+			}
+		} while (payload && *payload);
 
 		/* The add is done, send message. */
 		appctx->st0 = CLI_ST_PROMPT;
@@ -925,15 +979,17 @@ static int cli_parse_del_map(char **args, char *payload, struct appctx *appctx,
 	return 1;
 }
 
-
-/* continue to clear a map which was started in the parser */
+/* continue to clear a map which was started in the parser. The range of
+ * generations this applies to is taken from appctx->ctx.cli.i0 for the oldest
+ * and appctx->ctx.cli.i1 for the latest.
+ */
 static int cli_io_handler_clear_map(struct appctx *appctx)
 {
 	struct stream_interface *si = appctx->owner;
 	int finished;
 
 	HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-	finished = pat_ref_prune(appctx->ctx.map.ref);
+	finished = pat_ref_purge_range(appctx->ctx.map.ref, appctx->ctx.cli.i0, appctx->ctx.cli.i1, 100);
 	HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 
 	if (!finished) {
@@ -944,15 +1000,30 @@ static int cli_io_handler_clear_map(struct appctx *appctx)
 	return 1;
 }
 
+/* note: sets appctx->ctx.cli.i0 and appctx->ctx.cli.i1 to the oldest and
+ * latest generations to clear, respectively, and will call the clear_map
+ * handler.
+ */
 static int cli_parse_clear_map(char **args, char *payload, struct appctx *appctx, void *private)
 {
 	if (strcmp(args[1], "map") == 0 || strcmp(args[1], "acl") == 0) {
+		const char *gen = NULL;
+
 		/* Set ACL or MAP flags. */
 		if (args[1][0] == 'm')
 			appctx->ctx.map.display_flags = PAT_REF_MAP;
 		else
 			appctx->ctx.map.display_flags = PAT_REF_ACL;
 
+		/* For both "map" and "acl" we may have an optional generation
+		 * number specified using a "@" character before the pattern
+		 * file name.
+		 */
+		if (*args[2] == '@') {
+			gen = args[2] + 1;
+			args++;
+		}
+
 		/* no parameter */
 		if (!*args[2]) {
 			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
@@ -971,6 +1042,76 @@ static int cli_parse_clear_map(char **args, char *payload, struct appctx *appctx
 				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
 
+		/* set the desired generation id in cli.i0/i1 */
+		if (gen)
+			appctx->ctx.cli.i1 = appctx->ctx.cli.i0 = str2uic(gen);
+		else
+			appctx->ctx.cli.i1 = appctx->ctx.cli.i0 = appctx->ctx.map.ref->curr_gen;
+
+		/* delegate the clearing to the I/O handler which can yield */
+		return 0;
+	}
+	return 1;
+}
+
+/* note: sets appctx->ctx.cli.i0 and appctx->ctx.cli.i1 to the oldest and
+ * latest generations to clear, respectively, and will call the clear_map
+ * handler.
+ */
+static int cli_parse_commit_map(char **args, char *payload, struct appctx *appctx, void *private)
+{
+	if (strcmp(args[1], "map") == 0 || strcmp(args[1], "acl") == 0) {
+		const char *gen = NULL;
+		uint genid;
+		uint ret;
+
+		/* Set ACL or MAP flags. */
+		if (args[1][0] == 'm')
+			appctx->ctx.map.display_flags = PAT_REF_MAP;
+		else
+			appctx->ctx.map.display_flags = PAT_REF_ACL;
+
+		if (*args[2] != '@')
+			return cli_err(appctx, "Missing version number.\n");
+
+		/* The generation number is mandatory for a commit. The range
+		 * of generations that get trashed by a commit starts from the
+		 * opposite of the current one and ends at the previous one.
+		 */
+		gen = args[2] + 1;
+		genid = str2uic(gen);
+		appctx->ctx.cli.i1 = genid - 1;
+		appctx->ctx.cli.i0 = appctx->ctx.cli.i1 - ((~0U) >> 1);
+
+		/* no parameter */
+		if (!*args[3]) {
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Missing map identifier.\n");
+			else
+				return cli_err(appctx, "Missing ACL identifier.\n");
+		}
+
+		/* lookup into the refs and check the map flag */
+		appctx->ctx.map.ref = pat_ref_lookup_ref(args[3]);
+		if (!appctx->ctx.map.ref ||
+		    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags)) {
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
+		}
+
+		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+		if (genid - (appctx->ctx.map.ref->curr_gen + 1) <
+		    appctx->ctx.map.ref->next_gen - appctx->ctx.map.ref->curr_gen)
+			ret = pat_ref_commit(appctx->ctx.map.ref, genid);
+		else
+			ret = 1;
+		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+
+		if (ret != 0)
+			return cli_err(appctx, "Version number out of range.\n");
+
 		/* delegate the clearing to the I/O handler which can yield */
 		return 0;
 	}
@@ -980,17 +1121,21 @@ static int cli_parse_clear_map(char **args, char *payload, struct appctx *appctx
 /* register cli keywords */
 
 static struct cli_kw_list cli_kws = {{ },{
-	{ { "add",   "acl", NULL }, "add acl        : add acl entry", cli_parse_add_map, NULL },
-	{ { "clear", "acl", NULL }, "clear acl <id> : clear the content of this acl", cli_parse_clear_map, cli_io_handler_clear_map, NULL },
-	{ { "del",   "acl", NULL }, "del acl        : delete acl entry", cli_parse_del_map, NULL },
-	{ { "get",   "acl", NULL }, "get acl        : report the patterns matching a sample for an ACL", cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
-	{ { "show",  "acl", NULL }, "show acl [id]  : report available acls or dump an acl's contents", cli_parse_show_map, NULL },
-	{ { "add",   "map", NULL }, "add map        : add map entry", cli_parse_add_map, NULL },
-	{ { "clear", "map", NULL }, "clear map <id> : clear the content of this map", cli_parse_clear_map, cli_io_handler_clear_map, NULL },
-	{ { "del",   "map", NULL }, "del map        : delete map entry", cli_parse_del_map, NULL },
-	{ { "get",   "map", NULL }, "get map        : report the keys and values matching a sample for a map", cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
-	{ { "set",   "map", NULL }, "set map        : modify map entry", cli_parse_set_map, NULL },
-	{ { "show",  "map", NULL }, "show map [id]  : report available maps or dump a map's contents", cli_parse_show_map, NULL },
+	{ { "add",   "acl", NULL }, "add acl [@<ver>] <acl> <pattern>        : add an acl entry",                                       cli_parse_add_map, NULL },
+	{ { "clear", "acl", NULL }, "clear acl [@<ver>] <acl>                : clear the contents of this acl",                         cli_parse_clear_map, cli_io_handler_clear_map, NULL },
+	{ { "commit","acl", NULL }, "commit acl @<ver> <acl>                 : commit the ACL at this version",                         cli_parse_commit_map, cli_io_handler_clear_map, NULL },
+	{ { "del",   "acl", NULL }, "del acl <acl> [<key>|#<ref>]            : delete acl entries matching <key>",                      cli_parse_del_map, NULL },
+	{ { "get",   "acl", NULL }, "get acl <acl> <value>                   : report the patterns matching a sample for an ACL",       cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
+	{ { "prepare","acl",NULL }, "prepare acl <acl>                       : prepare a new version for atomic ACL replacement",       cli_parse_prepare_map, NULL },
+	{ { "show",  "acl", NULL }, "show acl [@<ver>] <acl>]                : report available acls or dump an acl's contents",        cli_parse_show_map, NULL },
+	{ { "add",   "map", NULL }, "add map [@<ver>] <map> <key> <val>      : add a map entry (payload supported instead of key/val)", cli_parse_add_map, NULL },
+	{ { "clear", "map", NULL }, "clear map [@<ver>] <map>                : clear the contents of this map",                         cli_parse_clear_map, cli_io_handler_clear_map, NULL },
+	{ { "commit","map", NULL }, "commit map @<ver> <map>                 : commit the map at this version",                         cli_parse_commit_map, cli_io_handler_clear_map, NULL },
+	{ { "del",   "map", NULL }, "del map <map> [<key>|#<ref>]            : delete map entries matching <key>",                      cli_parse_del_map, NULL },
+	{ { "get",   "map", NULL }, "get map <acl> <value>                   : report the keys and values matching a sample for a map", cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
+	{ { "prepare","map",NULL }, "prepare map <acl>                       : prepare a new version for atomic map replacement",       cli_parse_prepare_map, NULL },
+	{ { "set",   "map", NULL }, "set map <map> [<key>|#<ref>] <value>    : modify a map entry",                                     cli_parse_set_map, NULL },
+	{ { "show",  "map", NULL }, "show map [@ver] [map]                   : report available maps or dump a map's contents",         cli_parse_show_map, NULL },
 	{ { NULL }, NULL, NULL, NULL }
 }};
 
