@@ -27,6 +27,19 @@
 #include <haproxy/stream_interface.h>
 #include <haproxy/thread.h>
 
+/* Initialize a pre-allocated ring with the buffer area
+ * of size */
+void ring_init(struct ring *ring, void *area, size_t size)
+{
+	HA_RWLOCK_INIT(&ring->lock);
+	LIST_INIT(&ring->waiters);
+	ring->readers_count = 0;
+	ring->ofs = 0;
+	ring->buf = b_make(area, size, 0, 0);
+	/* write the initial RC byte */
+	b_putchr(&ring->buf, 0);
+}
+
 /* Creates and returns a ring buffer of size <size> bytes. Returns NULL on
  * allocation failure.
  */
@@ -46,13 +59,7 @@ struct ring *ring_new(size_t size)
 	if (!area)
 		goto fail;
 
-	HA_RWLOCK_INIT(&ring->lock);
-	LIST_INIT(&ring->waiters);
-	ring->readers_count = 0;
-	ring->ofs = 0;
-	ring->buf = b_make(area, size, 0, 0);
-	/* write the initial RC byte */
-	b_putchr(&ring->buf, 0);
+	ring_init(ring, area, size);
 	return ring;
  fail:
 	free(area);
@@ -84,7 +91,6 @@ struct ring *ring_resize(struct ring *ring, size_t size)
 		b_getblk(&ring->buf, area, ring->buf.data, 0);
 		area = HA_ATOMIC_XCHG(&ring->buf.area, area);
 		ring->buf.size = size;
-		ring->buf.head = 0;
 	}
 
 	HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
@@ -186,7 +192,7 @@ ssize_t ring_write(struct ring *ring, size_t maxlen, const struct ist pfx[], siz
 		totlen += len;
 	}
 
-	*b_tail(buf) = 0; buf->data++;; // new read counter
+	*b_tail(buf) = 0; buf->data++; // new read counter
 	sent = lenlen + totlen + 1;
 
 	/* notify potential readers */
@@ -230,9 +236,9 @@ void ring_detach_appctx(struct ring *ring, struct appctx *appctx, size_t ofs)
 		ofs -= ring->ofs;
 		BUG_ON(ofs >= b_size(&ring->buf));
 		LIST_DEL_INIT(&appctx->wait_entry);
-		HA_ATOMIC_SUB(b_peek(&ring->buf, ofs), 1);
+		HA_ATOMIC_DEC(b_peek(&ring->buf, ofs));
 	}
-	HA_ATOMIC_SUB(&ring->readers_count, 1);
+	HA_ATOMIC_DEC(&ring->readers_count);
 	HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
 }
 
@@ -302,7 +308,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 		if (appctx->ctx.cli.i0 & 2)
 			ofs += b_data(buf) - 1;
 
-		HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+		HA_ATOMIC_INC(b_peek(buf, ofs));
 		ofs += ring->ofs;
 	}
 
@@ -311,7 +317,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 	 */
 	ofs -= ring->ofs;
 	BUG_ON(ofs >= buf->size);
-	HA_ATOMIC_SUB(b_peek(buf, ofs), 1);
+	HA_ATOMIC_DEC(b_peek(buf, ofs));
 
 	/* in this loop, ofs always points to the counter byte that precedes
 	 * the message so that we can take our reference there if we have to
@@ -345,7 +351,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 		ofs += cnt + msg_len;
 	}
 
-	HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+	HA_ATOMIC_INC(b_peek(buf, ofs));
 	ofs += ring->ofs;
 	last_ofs = ring->ofs;
 	appctx->ctx.cli.o0 = ofs;
@@ -358,7 +364,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 		if (!si_oc(si)->output && !(si_oc(si)->flags & CF_SHUTW)) {
 			/* let's be woken up once new data arrive */
 			HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
-			LIST_ADDQ(&ring->waiters, &appctx->wait_entry);
+			LIST_APPEND(&ring->waiters, &appctx->wait_entry);
 			ofs = ring->ofs;
 			HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
 			if (ofs != last_ofs) {
