@@ -322,7 +322,7 @@ static void srv_state_srv_update(struct server *srv, int version, char **params)
 			srv_adm_set_drain(srv);
 	}
 
-	srv->last_change = date.tv_sec - srv_last_time_change;
+	srv->last_change = now.tv_sec - srv_last_time_change;
 	srv->check.status = srv_check_status;
 	srv->check.result = srv_check_result;
 
@@ -397,16 +397,12 @@ static void srv_state_srv_update(struct server *srv, int version, char **params)
 		char *tmp;
 
 		/* we can't apply previous state if SRV record has changed */
-		if (srv->srvrq && strcmp(srv->srvrq->name, srvrecord) != 0) {
-			chunk_appendf(msg, ", SRV record mismatch between configuration ('%s') and state file ('%s) for server '%s'. Previous state not applied", srv->srvrq->name, srvrecord, srv->id);
+		if (!srv->srvrq) {
+			chunk_appendf(msg, ", no SRV resolution for server '%s'. Previous state not applied", srv->id);
 			goto out;
 		}
-
-		/* create or find a SRV resolution for this srv record */
-		if (srv->srvrq == NULL && (srv->srvrq = find_srvrq_by_name(srvrecord, srv->proxy)) == NULL)
-			srv->srvrq = new_resolv_srvrq(srv, srvrecord);
-		if (srv->srvrq == NULL) {
-			chunk_appendf(msg, ", can't create or find SRV resolution '%s' for server '%s'", srvrecord, srv->id);
+		if (strcmp(srv->srvrq->name, srvrecord) != 0) {
+			chunk_appendf(msg, ", SRV record mismatch between configuration ('%s') and state file ('%s) for server '%s'. Previous state not applied", srv->srvrq->name, srvrecord, srv->id);
 			goto out;
 		}
 
@@ -429,8 +425,9 @@ static void srv_state_srv_update(struct server *srv, int version, char **params)
 		for (i = 0; tmp[i]; i++)
 			tmp[i] = tolower(tmp[i]);
 
-		/* insert in tree */
+		/* insert in tree and set the srvrq expiration date */
 		ebis_insert(&srv->srvrq->named_servers, &srv->host_dn);
+		task_schedule(srv->srvrq_check, tick_add(now_ms, srv->srvrq->resolvers->hold.timeout));
 
 		/* Unset SRV_F_MAPPORTS for SRV records.
 		 * SRV_F_MAPPORTS is unfortunately set by parse_server()
