@@ -114,7 +114,7 @@ void __trace(enum trace_level level, uint64_t mask, struct trace_source *src,
 
 	if (!sess && strm)
 		sess = strm->sess;
-	else if (!sess && conn)
+	else if (!sess && conn && LIST_INLIST(&conn->session_list))
 		sess = conn->owner;
 	else if (!sess && check)
 		sess = check->sess;
@@ -136,7 +136,7 @@ void __trace(enum trace_level level, uint64_t mask, struct trace_source *src,
 	}
 	if (check) {
 		srv = check->server;
-		be = srv->proxy;
+		be = (srv ? srv->proxy : NULL);
 	}
 
 	if (!srv && conn)
@@ -240,7 +240,7 @@ void __trace(enum trace_level level, uint64_t mask, struct trace_source *src,
 	}
 
 	if (src->sink)
-		sink_write(src->sink, line, words, 0, 0, NULL);
+		sink_write(src->sink, 0, line, words, 0, 0, NULL);
 
  end:
 	/* check if we need to stop the trace now */
