@@ -16,6 +16,7 @@
 #include <haproxy/debug.h>
 #include <haproxy/errors.h>
 #include <haproxy/global.h>
+#include <haproxy/signal-t.h>
 #include <haproxy/thread.h>
 #include <haproxy/tools.h>
 
@@ -65,8 +66,10 @@ void wdt_handler(int sig, siginfo_t *si, void *arg)
 		p = ha_thread_info[thr].prev_cpu_time;
 		n = now_cpu_time_thread(&ha_thread_info[thr]);
 
-		/* not yet reached the deadline of 1 sec */
-		if (n - p < 1000000000UL)
+		/* not yet reached the deadline of 1 sec,
+		 * or p wasn't initialized yet
+		 */
+		if (!p || n - p < 1000000000UL)
 			goto update_and_leave;
 
 		if ((threads_harmless_mask|sleeping_thread_mask|threads_to_dump) & (1UL << thr)) {
@@ -174,7 +177,7 @@ int init_wdt()
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_SIGINFO;
 	sigaction(WDTSIG, &sa, NULL);
-	return 0;
+	return ERR_NONE;
 }
 
 REGISTER_POST_CHECK(init_wdt);
